__docformat__ = 'restructuredtext'

from zope.i18n import translate
import time


def get_datepicker_date_format(request):
    """Return a localized date format we can use with the datepicker
    jqueryui plugin. The date format is retrieved from the
    date_format_short_datepicker msgid in the plonelocales i18n domain.
    Return 'mm/dd/yy' if no translation has been found.
    """
    date_format = translate(u"date_format_short_datepicker",
                            domain="plonelocales",
                            context=request)
    if date_format == u"date_format_short_datepicker":
        return 'mm/dd/yy'
    return date_format


def transform_to_percent(date_format):
    """Transform a jquery `date_format` to a date format that time.strptime
    understand. Replace "mm" by "%m", "dd" by "%d", and "yy" by "%Y".

    >>> transform_to_percent("mm/dd/yy")
    '%m/%d/%Y'
    >>> transform_to_percent("dd/mm/yy")
    '%d/%m/%Y'
    >>> transform_to_percent("yy/mm/dd")
    '%Y/%m/%d'
    >>> transform_to_percent("yy-mm-dd")
    '%Y-%m-%d'
    >>> transform_to_percent("dd.mm.yy")
    '%d.%m.%Y'
    >>> transform_to_percent("dd.mm.yy.")
    '%d.%m.%Y.'
    >>> transform_to_percent("yy.mm.dd.")
    '%Y.%m.%d.'
    """
    return date_format.replace(
        "mm", "%m").replace(
        "dd", "%d").replace(
        "yy", "%Y")


def get_python_date_format(request):
    return transform_to_percent(get_datepicker_date_format(request))


def parse_date(datestr, date_format):
    """Parse datestr given the date_format and return a tuple
    (year, month, day)

    >>> date = parse_date("17.03.2010", "%d.%m.%Y")
    >>> date
    (2010, 3, 17)

    And you can after that create a datetime object like this:

    >>> import datetime
    >>> datetime.date(*date)
    datetime.date(2010, 3, 17)
    """
    return time.strptime(datestr, date_format)[:3]



from Products.CMFPlone import interfaces as Plone
from Products.CMFQuickInstallerTool import interfaces as QuickInstaller
from collective.js.jqueryui.controlpanel import verify_jsregistry
from collective.js.jqueryui.interfaces import IJQueryUIPlugins
from plone.registry.interfaces import IRegistry
from zope.component._api import getUtility
from zope.interface import implementer


def applySettings(gscontext):
    # don't run as a step for other profiles
    if gscontext.readDataFile('jqueryui.txt') is None:
        return
    record = getUtility(IRegistry).forInterface(IJQueryUIPlugins)
    verify_jsregistry(record)


@implementer(Plone.INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Do not show on Plone's list of installable profiles.
        """
        return ['collective.js.jqueryui:install-base']


@implementer(QuickInstaller.INonInstallable)
class HiddenProducts(object):

    def getNonInstallableProducts(self):
        """Do not show on QuickInstaller's list of installable products.
        """
        return ['collective.js.jqueryui:install-base']



VERSION = '1.10.4'

DEPS = {}

DEPS['ui.core'] = tuple()
DEPS['ui.widget'] = tuple()
DEPS['ui.mouse'] = ('ui.core',)
DEPS['ui.position'] = tuple()

DEPS['ui.draggable'] = ('ui.core', 'ui.widget', 'ui.mouse')
DEPS['ui.droppable'] = ('ui.draggable', 'ui.core', 'ui.widget', 'ui.mouse')
DEPS['ui.resizable'] = ('ui.core', 'ui.widget', 'ui.mouse')
DEPS['ui.selectable'] = ('ui.core', 'ui.widget', 'ui.mouse')
DEPS['ui.sortable'] = ('ui.core', 'ui.widget', 'ui.mouse')

DEPS['ui.accordion'] = ('ui.core', 'ui.widget')
DEPS['ui.autocomplete'] = ('ui.core', 'ui.widget', 'ui.position', 'ui.menu')
DEPS['ui.button'] = ('ui.core', 'ui.widget')
DEPS['ui.datepicker'] = ('ui.core',)
DEPS['ui.dialog'] = ('ui.core', 'ui.widget', 'ui.position')
DEPS['ui.menu'] = ('ui.core', 'ui.widget', 'ui.position')
DEPS['ui.progressbar'] = ('ui.core', 'ui.widget')
DEPS['ui.slider'] = ('ui.core', 'ui.widget', 'ui.mouse')
DEPS['ui.spinner'] = ('ui.core', 'ui.widget', 'ui.button')
DEPS['ui.tabs'] = ('ui.core', 'ui.widget')
DEPS['ui.tooltip'] = ('ui.core', 'ui.widget', 'ui.position')

DEPS['ui.effect'] = tuple()
DEPS['ui.effect-blind'] = ('ui.effect',)
DEPS['ui.effect-bounce'] = ('ui.effect',)
DEPS['ui.effect-clip'] = ('ui.effect',)
DEPS['ui.effect-drop'] = ('ui.effect',)
DEPS['ui.effect-explode'] = ('ui.effect',)
DEPS['ui.effect-fade'] = ('ui.effect',)
DEPS['ui.effect-fold'] = ('ui.effect',)
DEPS['ui.effect-highlight'] = ('ui.effect',)
DEPS['ui.effect-pulsate'] = ('ui.effect',)
DEPS['ui.effect-scale'] = ('ui.effect',)
DEPS['ui.effect-shake'] = ('ui.effect',)
DEPS['ui.effect-slide'] = ('ui.effect',)
DEPS['ui.effect-transfer'] = ('ui.effect',)

JQUERYUI_DEPENDENCIES = DEPS

CSS_RESOURCE_ID = '++resource++jquery-ui-themes/sunburst/jqueryui.css'
PATCH_RESOURCE_ID = '++resource++jquery-ui-themes/sunburst-patch.css'

ORDERED_PLUGINS = ['ui.core', 'ui.widget', 'ui.mouse', 'ui.position',
                   'ui.draggable', 'ui.droppable', 'ui.resizable',
                   'ui.selectable', 'ui.sortable', 'ui.accordion',
                   'ui.autocomplete', 'ui.button',
                   'ui.datepicker', 'ui.dialog', 'ui.menu', 'ui.progressbar',
                   'ui.slider', 'ui.spinner', 'ui.tabs', 'ui.tooltip',
                   'ui.effect', 'ui.effect-blind',
                   'ui.effect-bounce', 'ui.effect-clip', 'ui.effect-drop',
                   'ui.effect-explode', 'ui.effect-fade', 'ui.effect-fold',
                   'ui.effect-highlight', 'ui.effect-pulsate',
                   'ui.effect-scale', 'ui.effect-shake', 'ui.effect-slide',
                   'ui.effect-transfer']



import logging
from copy import copy
from zope import component
from six import StringIO
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from Products.Five import BrowserView
from collective.js.jqueryui import interfaces, config

import os
import re

logger = logging.getLogger('collective.js.jqueryui')

URL_MATCH = re.compile(
    r'''(url\s*\(\s*['"]?)(?!data:)([^'")]+)(['"]?\s*\))''',
    re.I | re.S,
)


def makeAbsolute(path, prefix):
    """Make a url into an absolute URL by applying the given prefix
    """

    # Absolute path or full url
    if path.startswith('/') or '://' in path:
        return path

    absolute = "%s/%s" % (prefix, path)
    if '://' in absolute:
        return absolute

    normalized = os.path.normpath(absolute)
    if os.path.sep != '/':
        normalized = normalized.replace(os.path.sep, '/')
    return normalized


def applyPrefix(cssSource, prefix):
    """Return a copy of the string cssSource with each url() expression that
    contains an absolute path turned into an absolute path, by applying the
    given prefix.
    """
    if prefix.endswith('/'):
        prefix = prefix[:-1]

    return URL_MATCH.sub(
        lambda m: m.group(1) + makeAbsolute(m.group(2), prefix) + m.group(3),
        cssSource,
    )


class Resources(BrowserView):
    _key = None
    _header_template = u"\n/* collective.js.jqueryui: %s */\n"
    _mimetype = None
    _tool = None
    _toolid = None
    _packer = None
    _js = False
    _css = False

    @property
    def tool(self):
        if self._tool is None:
            if self._toolid:
                self._tool = getToolByName(self.context,
                                           self._toolid)
            else:
                self._tool = False
        return self._tool

    @property
    def debug(self):
        ret = False
        if self.tool:
            ret = self.tool.getDebugMode()
        return ret

    def settings(self):
        """Return records from portal_registry corresponding to
        IJQueryUIPlugins
        """
        registry = component.queryUtility(IRegistry)
        if registry is None or self._key is None:
            return
        return registry.forInterface(self._key, None)

    def __call__(self):
        settings = self.settings
        if settings is None:
            return
        resources = self.get_resources()
        if self._mimetype:
            self.request.response.setHeader('Content-Type',
                                            self._mimetype)
        if resources is None:
            resources = ""
        return self.get_resources_content(resources)

    def pack(self, content):
        if self._packer and not self.debug:
            return self._packer('safe').pack(content)
        return content

    def get_resources_content(self, resources):
        """Resources must be a list of resource ids.
        This method return the content of each resources appended in one string
        """
        data = StringIO()
        if resources is None:
            resources = self.get_resources()
        upath = self.context.absolute_url_path()
        for resourceid in resources:
            try:
                resource = self.context.restrictedTraverse(
                    resourceid)
            except KeyError as e:
                logger.error(e)
                continue
            if not resource:
                continue
            if self._header_template:
                data.write(
                    self._header_template % (
                        resource.context.__name__
                    )
                )
                data.write(u"\n")

            # read the content of the resource
            fic = open(resource.context.path, 'r')
            content = fic.read()
            fic.close()
            content = safe_unicode(content)

            # css applyPrefix
            if self._css:
                upath = upath.rstrip('/')
                prefix = '/'.join([upath, resourceid])
                if prefix.endswith('/'):
                    prefix = prefix[:-1]
                prefix = '/'.join(
                    prefix.split('/')[:-1])
                content = applyPrefix(content, prefix)

            # content is already minified
            data.write(content)
            data.write(u"\n")

        return data.getvalue()


class JQueryUICustomJS(Resources):
    """This view aggregate javascripts according to the configuration. It has
    been created to not polute the portal_javascripts will all plugins"""
    _js = True
    _css = False
    _key = interfaces.IJQueryUIPlugins
    _mimetype = 'application/javascript'
    _toolid = 'portal_javascripts'
    # _packer = JavascriptPacker

    def get_resources(self):
        """Return a list of resources (at least their ids) computed from
        the configuration provided by IJQueryUIPlugins's records in plone
        portal_registry"""
        settings = self.settings()
        deps = config.JQUERYUI_DEPENDENCIES
        alljs = list(deps.keys())
        resources = []
        wanted = []  # not ordered list of wanted plugins
        tpl = "++resource++jquery-ui/jquery.%s.min.js"
        ordered_plugins = copy(config.ORDERED_PLUGINS)
        for plugin in alljs:
            attr_name = plugin.replace('.', '_').replace('-', '_')
            is_wanted = getattr(settings, attr_name, False)
            if is_wanted:
                wanted.append(plugin)
                deps = config.JQUERYUI_DEPENDENCIES[plugin]
                for dep in deps:
                    wanted.append(dep)
        for plugin in alljs:
            if plugin not in wanted:
                ordered_plugins.remove(plugin)
        for plugin in ordered_plugins:
            resources.append(tpl % plugin)
            if plugin == 'ui.datepicker':
                resources.append('++resource++jquery-ui-i18n.js')
        return resources


class JQueryUICustomCSS(Resources):
    """ . """
    _key = interfaces.IJQueryUICSS
    _mimetype = 'text/css'
    _toolid = 'portal_css'
    # _packer = CSSPacker
    _js = False
    _css = True

    def get_resources(self):
        """Return a list of resources (at least their ids) computed from
        the configuration provided by IJQueryUIPlugins's records in plone
        portal_registry"""
        settings = self.settings()
        resources_items = [
            ("css", config.CSS_RESOURCE_ID),
            ("patch", config.PATCH_RESOURCE_ID)]
        resources = []
        for skey, cssid in resources_items:
            if getattr(settings, skey, None):
                resources.append(cssid)
        return resources



from setuptools import setup, find_packages
import os

version = '2.1.2'

setup(
    name='collective.js.jqueryui',
    version=version,
    description="JQueryUI ready for Plone",
    long_description=open("README.rst").read() + "\n" +
         open(os.path.join("docs", "UPGRADE.txt")).read() + "\n" +
         open("CHANGES.rst").read(),
    classifiers=[
        "Environment :: Web Environment",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
        "Framework :: Zope2",
        "Framework :: Zope :: 2",
        "Framework :: Zope :: 4",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 5.0",
        "Framework :: Plone :: 5.1",
        "Framework :: Plone :: 5.2",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    keywords='plone jqueryui',
    author='JeanMichel FRANCOIS aka toutpt',
    author_email='toutpt@gmail.com',
    url='https://github.com/collective/collective.js.jqueryui',
    license='GPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['collective', 'collective.js'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'plone.app.jquery > 1.6',
        'Products.CMFPlone',
        'setuptools',
    ],
    setup_requires=["setuptools_git"],
    extras_require={},
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
